from QAManual import FunctionParser, ClassParser
from QAManual import MarkdownWriter
from tests.cccc import example_func

class QA:
    def example_func(self, code, source):
        """QAMAN
        explanation:
            此函数用于在ctp和通达信之间来回转换

        params:
            * code ->
                含义: 测试用例
                类型: str
                参数支持: []
            * source ->
                含义: 需要转换到的目标
                类型: List
                参数支持: ["pytdx", "ctp"]

        demonstrate:
            example_func("你好", "somewheve")

        output:
            >>somewheve --> 你好
        """
        print(source, "-->", code)




if __name__ == '__main__':
    parser = ClassParser(QA)
    for x in parser.get_node():
        writer = MarkdownWriter(x)
        writer.handle()



import re
from collections import namedtuple

from QAManual.bash_parser import ParamsParser
from QAManual.c import meaning, get_key


class Doc:
    def __init__(self, node: namedtuple, language: str):
        """ parse node """

        self.node = node
        self.lng = language
        self.params = ""
        self.name = ""
        self.output = ""
        self.explanation = ""
        self.demonstrate = ""
        self._next_()

    def _next_(self) -> None:
        for x in self.node.doc:
            for cval in meaning[self.lng]['level_pri'].values():
                if get_key(meaning[self.lng]['level_pri'], cval) in x:
                    value = re.sub(f"{get_key(meaning[self.lng]['level_pri'], cval)}(.*)", lambda x: x.group(1), x)
                    setattr(self,
                            get_key(meaning[self.lng]['level_pri'], cval).replace(":", ""),
                            value)
                else:
                    continue
        self.name = self.node.name
        self.param_formatter = ParamsParser(self.params, self.lng)



def example_func(code, source):
    """
    QAMAN
    explanation:
        此函数用于在ctp和通达信之间来回转换

    params:
        * code ->
            含义: 测试用例
            类型: str
            参数支持: []
        * source ->
            含义: 需要转换到的目标
            类型: List
            参数支持: ["pytdx", "ctp"]

    demonstrate:
        example_func("你好", "somewheve")

    output:
        >>somewheve --> 你好
    """
    print(source, "-->", code)



"""
 内置基础解析器
"""
import inspect
import re
import typing
import types
from copy import deepcopy
from collections import namedtuple

from QAManual.c import meaning, get_key

doc_node = namedtuple("code_node", ["name", "code_source", "code_file", "doc"])


class Parser:

    def __init__(self, destination, origin_name=None, ):
        if inspect.isfunction(destination):
            self.type = "func"
        if inspect.isclass(destination):
            self.type = "class"
        self.union = destination
        self.name = self.union.__name__
        self.origin_name = origin_name

    @property
    def doc(self):
        if len(self.union.__doc__) == 0 or "QAMAN" not in self.union.__doc__:
            return None
        return self.union.__doc__.replace("QAMAN", "")

    @property
    def code_source(self):
        try:
            return inspect.getsource(self.union)
        except OSError:
            return " "

    @property
    def code_file(self):
        p = inspect.getsourcefile(self.union)
        return p

    def parse(self):
        """override this method to perform parsing structure"""
        raise NotImplemented

    def get_node(self) -> doc_node or [doc_node]:
        """get structured data"""
        if self.type == "func":
            if self.origin_name:
                name = f"{self.origin_name}.{self.union.__name__}"
            else:
                name = self.union.__name__
            print("正在解析 --->", name)
            return doc_node(name=name, doc=self.parse(),
                            code_source=self.code_source,
                            code_file=self.code_file)
        elif self.type == "class":
            return self.parse()


class FunctionParser(Parser):

    def parse(self) -> typing.Mapping or None:
        if self.doc:
            return [x for x in self.doc.split("\n\n")]
        else:
            return None


class ClassParser(Parser):
    def parse(self):
        """  调用函数进行解析, 返回多个node """
        result = []

        for x in self.union.__dict__.values():
            if inspect.isfunction(x):
                result.append(FunctionParser(x, self.union.__name__).get_node())
            else:
                continue
        return result


class ParamsParser:
    """ format for parameters """

    def __init__(self, source: str, language: str):
        self.source = source
        self.lng = language
        self.name = set()
        self.attr = self._next_()

    def _next_(self):
        """ parse parameters and generate structure """
        rtemp = re.split("(\*\s\w+\s\->)", self.source)
        result = {}
        for ink in range(len(rtemp)):
            element = rtemp[ink]
            if re.match("\*\s\w+\s\->", element) is not None:
                next_element = rtemp[ink + 1]
                temp_element = deepcopy(next_element)
                self.attr_name = re.match("^\*\s(\w+)\s\->", element).group(1)
                for val in meaning[self.lng]['level_sec'].values():
                    key = get_key(meaning[self.lng]['level_sec'], val)
                    self.name.add(key)
                    if val in temp_element:
                        pattern = re.compile("^\n[\w\W]*" + f"{val}:" + r"[\s]?(.*?)\n")
                        # Add a newline manually to ensure that the last line can be matched
                        c = pattern.match(temp_element + "\n")
                        if c:
                            result.setdefault(self.attr_name, {})[key] = c.group(1)
            else:
                continue
        return result



"""
需要支持中文和英文两种注解生成. 支持更多语言
"""

meaning = {
    "zh":
        {
            "level_pri":
                {
                    "params:": "参数",
                    "demonstrate:": "调用示例",
                    "output:": "输出",
                    "explanation:": "用途"
                },
            "level_sec":
                {
                    "meaning": "含义",
                    "type": "类型",
                    "optional_value": "参数支持"
                },
            "level_mean":
                {
                    "source_code": "源代码",
                    "explanation": "解释",
                    "output": "示例输出",
                    "demonstrate": "调用示例",
                    "path": "路径"
                }
        },
    "en":
        {
            "level_pri":
                {
                    "params:": "params",
                    "demonstrate:": "demonstrate",
                    "output:": "output",
                    "explanation:": "explanation"
                },
            "level_sec":
                {
                    "meaning": "meaning",
                    "type": "type",
                    "optional_value": "optional_value"
                },
            "level_mean":
                {
                    "source_code": "Origin source",
                    "explanation": "Explanation",
                    "output": "Example output",
                    "demonstrate": "Call example",
                    "path": "Path"
                }
        },

}


def get_key(obj, value):
    for key, val in obj.items():
        if val == value:
            return key
    else:
        return None



"""
This project are using to support python project documentation
"""

from QAManual.writer import RstWriter, MarkdownWriter
from QAManual.doc_parse import Doc
from QAManual.bash_parser import FunctionParser, ClassParser, Parser



"""
命令行搜索程序
"""
import argparse
import os
import sys
import types
import inspect
from .writer import MarkdownWriter
from .bash_parser import FunctionParser, ClassParser

parser = argparse.ArgumentParser(description="Let's go! Man~")
parser.add_argument('-init', '--init', help='project initialization command, ;'
                                            'qaman --init .')
parser.add_argument('-o', '--output', help='output the address of the generated document;'
                                           'qaman -init . -o /home/somewheve/au .')
parser.add_argument("-s", '--search', help="Enter keywords, query API, use with project parameters")
parser.add_argument("-path", '--abspath', help="add absolute search path")
parser.add_argument("-l", '--language', default="zh", help="specify language")
parser.add_argument("-p", '--project',
                    help="Incoming project nouns, functions and functions for querying specified projects in projects ")


class QAMan:

    @classmethod
    def search(cls, args):
        """
        search command
        """

    @classmethod
    def init(cls, args):
        env = {}
        for key, value in locals().items():
            env[key] = value
        cwd = os.getcwd()
        if args.init == ".":
            file_path = os.path.join(cwd, "__init__.py")
        else:
            file_path = os.path.join(args.abspath, "__init__.py")
        if not os.path.exists(file_path):
            print("\n--->当前路径不存在__init__.py文件,请切换到项目根目录下面执行此命令,或者通过-path参数直接添加 \n--->程序终止!!!\n")
            return
        with open(file_path, "r+") as f:
            d = types.ModuleType("object")
            d.__file__ = f.name
            exec(compile(f.read(), f.name, 'exec'), d.__dict__)
        for k, v in d.__dict__.items():
            if k in env.keys():
                continue
            else:
                if inspect.isfunction(v):
                    """ Processing function """
                    parse = FunctionParser(v)
                    writer = MarkdownWriter(parse.get_node(), cwd, path=args.output, language=args.language)
                    writer.handle()
                elif inspect.isclass(v):
                    """ Processing class """
                    try:

                        parse = ClassParser(v)
                        for x in parse.get_node():
                            writer = MarkdownWriter(x, cwd, path=args.output, language=args.language)
                            writer.handle()
                    except TypeError:
                        continue
                else:
                    continue
        print("\n--->执行成功 !!\n")


def main():
    args = parser.parse_args()
    if args.init is not None:
        """ processing initialization commands """
        QAMan.init(args)
    if args.search is not None:
        """ processing query commands """
        QAMan.search(args)



from setuptools import setup, find_packages

setup(
    name="QAManual",
    author="somewheve",
    version="0.2",
    packages=find_packages(),
    entry_points={
        'console_scripts': ['qaman=QAManual.command:main'],
    }

)



"""
写入markdown模块
注意你需要使用标记
"""
import os
import re
import sys

from QAManual.bash_parser import ParamsParser
from QAManual.doc_parse import Doc
from QAManual.c import meaning


class Writer:
    exc = ""

    def __init__(self, node, cwd=os.getcwd(), path=None, language="zh"):
        self.cwd = cwd
        self.node = node
        self.doc_result = Doc(node, language)
        self.params = ParamsParser(self.doc_result.params, language)
        # source code file path
        code_filepath = self.node.code_file
        file_name = code_filepath.replace(cwd, "")
        if not path:
            """ if the specified output path is passed in """
            destination_dir = os.path.join(cwd, "doc_output")
        else:
            """如果传入了指定文件夹路径"""
            if os.path.exists(path):
                """ 绝对路径 """
                destination_dir = path
            else:
                """ 相对路径 """
                destination_dir = os.path.join(cwd, path)
        # create folder
        if not os.path.exists(destination_dir):
            os.mkdir(destination_dir)
        self.filename = os.path.join(destination_dir, re.sub(r"[/\\]+(.*)", lambda x: x.group(1), file_name)).replace(
            ".py", ".md")
        self.lng = language

    def write(self, content):
        """basic writing method"""
        with open(self.filename, "a+", encoding="utf-8") as f:
            f.write(content)

    def search(self):
        raise NotImplemented

    def code_writer(self):
        """ write code object """
        raise NotImplemented

    def explanation_writer(self):
        raise NotImplemented

    def name_writer(self):
        """ function name write method"""
        raise NotImplemented

    def params_writer(self):
        """ parameter writing method """
        raise NotImplemented

    def output_writer(self):
        """ output writing method """
        raise NotImplemented

    def demonstrate_writer(self):
        """ example write method """
        raise NotImplemented

    def trim(self, docstring):
        """ unindent doc"""
        if not docstring:
            return ''
        lines = docstring.expandtabs().splitlines()
        indent = sys.maxsize
        for line in lines[1:]:
            stripped = line.lstrip()
            if stripped:
                indent = min(indent, len(line) - len(stripped))
        trimmed = [lines[0].strip()]
        if indent < sys.maxsize:
            length = len(lines[1:])
            for ind in range(len(lines[1:])):
                content = lines[1:][ind]
                trimmed.append(content[indent:] + "\n") if ind != length - 1 else trimmed.append(content[indent:])
        while trimmed and not trimmed[-1]:
            trimmed.pop()
        while trimmed and not trimmed[0]:
            trimmed.pop(0)
        return ''.join(trimmed)

    def handle(self):
        """ standard processing method """


class MarkdownWriter(Writer):
    """
    markdown document writer
    """

    def name_writer(self):
        self.write("##### " + self.doc_result.name + "\n")

    def code_writer(self):
        self.write("```python\n" + self.node.code_source + "\n```")

    def explanation_writer(self):
        self.write("> ".join([x + "\n" for x in self.trim(self.doc_result.explanation).split("\n")]) + "\n" + "\n")

    def output_writer(self):
        self.write(
            f"{meaning[self.lng]['level_mean']['output']}" + "\n" + "```\n" + self.trim(
                self.doc_result.output) + "\n```" + "\n")

    def demonstrate_writer(self):
        self.write(f"**{meaning[self.lng]['level_mean']['demonstrate']}**" + "\n" + "```\n" + self.trim(
            self.doc_result.demonstrate) + "\n```" + "\n")

    def params_writer(self):
        length = len(self.params.name)

        def generate_header():
            """ generate doc header of the markdown table"""
            return "| Attr | " + " |".join(self.params.name) + "| \n" + "|" + "|".join(
                [":----:" for x in range(length + 1)]) + "|\n"

        header = "**Attr Table**\n\n" + generate_header()
        body = ""
        for key, value in self.params.attr.items():
            body += ("|" + key)
            for x in self.params.name:
                body += "|" + str(self.params.attr[key].get(x, None))
            body += "|\n"
        end = "\n" + header + body + "\n"
        self.write(end)

    def path_writer(self):
        self.write(f"{meaning[self.lng]['level_mean']['path']}: `{self.node.code_file}`" + "\n")

    def handle(self, code=False):
        self.name_writer()
        self.path_writer()
        self.explanation_writer()
        self.params_writer()
        self.demonstrate_writer()
        self.output_writer()
        if code:
            self.code_writer()


class RstWriter(Writer):
    """
    rst document writer
    """
